import { I as IShellMessage, P as PkgTool, N as NodeWorkerPayload } from '../types-82873c18.js';
import { StdioOptions, SpawnOptions } from 'node:child_process';

/**
 * 将文件拷贝到另一个目录
 * @param urls 需要被拷贝的文件路径
 * @param destination 目录
 * @returns IShellMessage
 */
declare function fileCopy(urls: string[], destination: string): Promise<IShellMessage>;

/**
 * 获取npm包导出的文件内容
 * @param { string } url npm包名
 * @returns
 */
declare function getExportBundle(url: string): Promise<string>;

interface IPackage {
    name: string;
    version: string;
    description?: string;
    main?: string;
    module?: string;
    types?: string;
    files?: string[];
    scripts?: Record<string, string>;
}
/**
 * 获取当前package.json 对象
 * @param { string } url 路径 默认 ./package.json
 * @returns 返回json格式package.json
 */
declare function getPkg(url?: string): Promise<IPackage & Record<string, any>>;

/**
 * 获取当前包管理器 ‘yarn’ | 'pnpm' | 'bun' | 'npm'
 * @returns 返回当前package环境 ‘yarn’ | 'pnpm' | 'bun' | 'npm'
 */
declare function getPkgTool(): Promise<PkgTool>;

/**
 *
 * @param { string | string[] } commander 指令
 * @param { string } stdio 'inherit' | 'pipe'
 * @param { boolean } errorExit 错误时是否结束进程
 * @returns
 */
interface Options {
    args?: string[];
    stdio?: StdioOptions | undefined;
    errorExit?: boolean;
    isLog?: boolean;
    cwd?: string;
    options?: SpawnOptions;
}
declare function jsShell<T extends string | string[]>(commander: T, options?: Options | Options['stdio']): T extends string ? Promise<IShellMessage> : Promise<IShellMessage[]>;

/**
 *
 * @returns 处理argv --flag如果未设置值默认为true
 */
declare function transformArgv(): Record<string, any>;

type NodeWorkReturn<T> = T extends {
    params: string[];
    stdio?: 'inherit' | 'pipe';
} ? IShellMessage[] : IShellMessage;
/**
 * 这个api需要 dependencies 中安装lazy-js-utils
 * @param { string | NodeWorkerPayload } payload 字符串 ｜ {
  params: string[]
  stdio?: 'inherit' | 'pipe'
}
 * @param { string } [url] 自定义worker路径
 * @returns
 */
declare function useNodeWorker<T extends NodeWorkerPayload | string>(payload: T, url?: string): Promise<NodeWorkReturn<T>>;
declare function useProcressNodeWorker(callback: (data: any) => any): void;

/**
 * 处理gulp任务
 * @param name 任务名
 * @param fn 任务函数
 * @returns
 */
declare function withTaskName<T extends Function>(name: string, fn: T): T & {
    displayName: string;
};

/**
 * 重写文件
 * @param { string } paths 路径
 * @param { (content: string, index: number) => string } callback 回调接收文件字符串将返回的内容重新写入该文件
 * @param { string } [encoding] 默认 'utf-8'
 */
declare function writeFile(paths: string[] | string, callback: (content: string, index: number) => string, encoding?: BufferEncoding): void;

/**
 * 判断是否存在package.json
 * @param { string } rootPath 绝对路径
 * @returns boolean
 */
declare function hasPkg(rootPath: string): Promise<boolean>;

declare function isInstallPkg(pkg: string): Promise<boolean>;

export { fileCopy, getExportBundle, getPkg, getPkgTool, hasPkg, isInstallPkg, jsShell, transformArgv, useNodeWorker, useProcressNodeWorker, withTaskName, writeFile };
