// src/node/jsShell.ts
import child_process from "child_process";
import process from "process";

// src/is/isArray.ts
var isArray = Array.isArray;

// src/node/jsShell.ts
function jsShell(commander, options) {
  let args = [];
  let stdio;
  let errorExit = true;
  let isLog = false;
  let cwd;
  let _options = {};
  if (typeof options === "string") {
    stdio = options;
  } else if (options) {
    args = options.args ?? [];
    stdio = options.stdio;
    errorExit = options.errorExit ?? true;
    isLog = options.isLog ?? false;
    cwd = options.cwd;
    _options = options.options ?? {};
  }
  return isArray(commander) ? Promise.all(commander.map((command) => executor(command))) : executor(commander);
  function executor(commander2) {
    return new Promise((resolve, reject) => {
      var _a, _b;
      const child = child_process.spawn(commander2, args, {
        shell: true,
        stdio,
        cwd,
        ..._options
      });
      const pid = child.pid;
      let result = "";
      (_a = child.stdout) == null ? void 0 : _a.on("data", (data) => {
        result += data.toString();
      });
      (_b = child.stderr) == null ? void 0 : _b.on("data", (data) => {
        if (isLog)
          console.error(data.toString());
      });
      child.on("close", (status) => {
        result = result.trim();
        if (status === 130) {
          if (isLog)
            console.log("\u5DF2\u53D6\u6D88...");
          resolve({ status, result, pid });
        } else if (status !== 0) {
          if (isLog)
            console.error(`Command failed with status ${status}`);
          if (errorExit) {
            reject(new Error(result));
            process.exit(1);
          } else {
            resolve({ status, result, pid });
          }
        } else {
          resolve({ status, result, pid });
        }
      });
      child.on("error", (error) => {
        if (isLog)
          console.error(error.message);
        if (errorExit) {
          reject(new Error(error.message));
          process.exit(1);
        } else {
          resolve({ status: null, result: error.message, pid });
        }
      });
    });
  }
}

// src/node/fileCopy.ts
function fileCopy(urls, destination) {
  return jsShell(`cp -r {${urls.join(",")}} ${destination}`, "pipe");
}

// src/node/getExportBundle.ts
import fsp from "fs/promises";
import path from "path";
import process2 from "process";
async function getExportBundle(url) {
  if (/[./]/.test(url))
    throw new Error("module must be a npm module");
  const pkg = path.resolve(process2.cwd(), "node_modules", url, "package.json");
  const { module, main } = JSON.parse(await fsp.readFile(pkg, "utf-8"));
  const modulePath = path.resolve(`./node_modules/${url}`, module || main);
  return fsp.readFile(modulePath, "utf-8");
}

// src/node/getPkg.ts
import fsp2 from "fs/promises";

// src/is/isFile.ts
import fs from "fs";

// src/utils/common.ts
var _toString = Object.prototype.toString;

// src/is/isStr.ts
function isStr(o) {
  return typeof o === "string";
}

// src/is/isFile.ts
function isFile(o) {
  if (isStr(o)) {
    try {
      return fs.statSync(o).isFile();
    } catch (error) {
      return false;
    }
  }
  return _toString.call(o) === "[object File]";
}

// src/string/ensureSuffix.ts
function ensureSuffix(suffix, str) {
  if (str.endsWith(suffix))
    return str;
  return str + suffix;
}

// src/to/toAbsolutePath.ts
import process3 from "process";
import path2 from "path";

// src/is/isAbsolute.ts
function isAbsolute(url) {
  return /^\/|^\\|^[a-z]:[/\\]/i.test(url);
}

// src/to/toAbsolutePath.ts
function toAbsolutePath(url) {
  return isAbsolute(url) ? url : path2.resolve(process3.cwd(), url);
}

// src/node/getPkg.ts
async function getPkg(url = "./package.json") {
  const resolvedPath = toAbsolutePath(ensureSuffix("/package.json", url));
  if (!isFile(resolvedPath))
    throw new Error(`${resolvedPath} is not a file`);
  return JSON.parse(await fsp2.readFile(resolvedPath, "utf-8"));
}

// src/node/getPkgTool.ts
async function getPkgTool() {
  const pkg = await getPkg() || {};
  const { packageManager } = pkg;
  if (packageManager) {
    const manager = packageManager.split("@")[0].replace(/[~^]/, "");
    if (packageManager)
      return manager;
  }
  switch (true) {
    case isFile(toAbsolutePath("./bun.lockb")):
      return "bun";
    case isFile(toAbsolutePath("./pnpm-lock.yaml")):
    case isFile(toAbsolutePath("./pnpm-workspace.yaml")):
      return "pnpm";
    case isFile(toAbsolutePath("./yarn.lock")):
    case isFile(toAbsolutePath("./lerna.json")):
      return "yarn";
    default:
      return "npm";
  }
}

// src/node/transformArgv.ts
import process4 from "process";
function transformArgv() {
  var _a, _b;
  return (_b = (_a = process4) == null ? void 0 : _a.argv) == null ? void 0 : _b.slice(2).reduce((result, arg) => {
    const [key, value] = arg.split("=");
    result[key.slice(2)] = value || true;
    return result;
  }, {});
}

// src/node/useNodeWorker.ts
import worker_threads from "worker_threads";
import path3 from "path";
import process5 from "process";
import fs2 from "fs";

// src/js/parallel.ts
function parallel(tasks, fn) {
  return Promise.all(tasks.map(fn));
}

// src/node/useNodeWorker.ts
async function useNodeWorker(payload, url) {
  if (!url) {
    url = path3.resolve(
      __dirname,
      "../node_modules/lazy-js-utils/dist/worker/useNodeWorkerThread.mjs"
    );
    if (!fs2.existsSync(url))
      url = path3.resolve(__dirname, "../worker/useNodeWorkerThread.mjs");
  }
  const { params } = isStr(payload) ? payload = { params: payload } : payload;
  const commands = isArray(params) ? params : params.split("&&");
  const result = await parallel(commands, (params2) => createWorker(
    Object.assign(payload, {
      params: params2
    })
  ));
  setTimeout(process5.exit);
  return result.length === 1 ? result[0] : result;
  function createWorker(payload2) {
    const { Worker } = worker_threads;
    return new Promise((resolve) => {
      const seprateThread = new Worker(url);
      seprateThread.on("message", resolve);
      seprateThread.postMessage(payload2);
    });
  }
}
function useProcressNodeWorker(callback) {
  const { parentPort } = worker_threads;
  parentPort.on("message", async (data) => parentPort == null ? void 0 : parentPort.postMessage(await (callback == null ? void 0 : callback(data)) || (() => "")));
}

// src/node/withTaskName.ts
function withTaskName(name, fn) {
  return Object.assign(fn, { displayName: name });
}

// src/node/writeFile.ts
import fsp3 from "fs/promises";
function writeFile(paths, callback, encoding = "utf-8") {
  if (isStr(paths))
    paths = [paths];
  paths.forEach(async (relativepath, i) => {
    const content = await fsp3.readFile(relativepath, encoding);
    const result = (callback == null ? void 0 : callback(content, i)) || content;
    fsp3.writeFile(relativepath, result).catch((err) => {
      throw err;
    });
  });
}

// src/node/hasPkg.ts
import path4 from "path";
async function hasPkg(rootPath) {
  const url = path4.resolve(rootPath, "package.json");
  const { result } = await jsShell(
    `test -f "${url}" && echo "0"|| echo "1"`,
    "pipe"
  );
  return result === "0";
}

// src/node/isInstallPkg.ts
async function isInstallPkg(pkg) {
  const { status } = await jsShell(`if ! command -v ${pkg} &> /dev/null; then
  exit 1
else
  exit 0
fi`);
  return status === 0;
}
export {
  fileCopy,
  getExportBundle,
  getPkg,
  getPkgTool,
  hasPkg,
  isInstallPkg,
  jsShell,
  transformArgv,
  useNodeWorker,
  useProcressNodeWorker,
  withTaskName,
  writeFile
};
