// src/is/isArray.ts
var isArray = Array.isArray;

// src/is/isBool.ts
function isBool(value) {
  return typeof value === "boolean";
}

// src/is/isNull.ts
function isNull(o) {
  return o === null;
}

// src/event/findElement.ts
function findElement(selector, all = false, currentNode = document) {
  if (isNull(all))
    return;
  if (!isBool(all)) {
    currentNode = all;
    all = false;
  }
  if (isArray(selector)) {
    return selector.reduce((result, c) => {
      const item = all ? currentNode.querySelectorAll(c) : currentNode.querySelector(c);
      if (!item)
        return result;
      return result = all ? [...result, ...item] : [...result, item];
    }, []);
  }
  return all ? currentNode.querySelectorAll(selector) : currentNode.querySelector(selector);
}

// src/is/isStr.ts
function isStr(o) {
  return typeof o === "string";
}

// src/utils/mount.ts
function mount(...args) {
  const len = args.length;
  const params = [...args];
  const elements = params.slice(0, len - 1);
  const callback = params.slice(-1)[0];
  let isMounted = false;
  let hasMounted = false;
  update();
  document.addEventListener("DOMContentLoaded", update);
  setTimeout(() => document.removeEventListener("DOMContentLoaded", update));
  function update() {
    if (hasMounted)
      return;
    elements.forEach(
      (element, index) => isStr(element) && (elements[index] = findElement(element) || element)
    );
    if (!isMounted && elements.some(isStr))
      return isMounted = true;
    if (elements.some(isStr)) {
      throw new Error(
        `${elements.filter(isStr).join(", ")} is not a HTMLElement`
      );
    }
    callback == null ? void 0 : callback(...elements);
    hasMounted = true;
  }
}

// src/utils/unmount.ts
function unmount(callback) {
  const fn = window.onunload || function() {
  };
  window.onunload = function(ev) {
    callback == null ? void 0 : callback(ev);
    fn.call(this, ev);
  };
  return callback;
}

// src/event/useMutationObserver.ts
function useMutationObserver(element, callback, options = { childList: true }) {
  if (!element)
    return;
  let stopped = false;
  let stop;
  mount(element, (element2) => {
    const mutationObserver = new MutationObserver(callback);
    mutationObserver.observe(element2, options);
    stop = () => mutationObserver.disconnect();
    if (stopped)
      stop();
  });
  unmount(() => stop == null ? void 0 : stop());
  return () => {
    if (!stop)
      return stopped = true;
    stop == null ? void 0 : stop();
  };
}

// src/webComponent/render.ts
function render(arr, node) {
  arr = isArray(arr) ? arr : [arr];
  arr.forEach((item) => {
    const el = document.createElement(item.tag);
    if (item.class)
      el.className = item.class;
    if (item.name)
      el.setAttribute("name", item.name);
    if (typeof item.children === "string") {
      const text = document.createTextNode(item.children);
      el.appendChild(text);
    } else if (item.children) {
      render(item.children, el);
    }
    node.appendChild(el);
  });
}
var Render = class extends HTMLElement {
  props = {};
  /**
   * 创建影子节点
   * https://developer.mozilla.org/zh-CN/docs/Web/API/Element/attachShadow
   */
  shadowRoot = this.attachShadow({ mode: "open" });
  constructor() {
    super();
    this.initial();
    this.setupShadow();
  }
  initial() {
    useMutationObserver(this, () => this.setProps(), {
      attributes: true
    });
  }
  /**
   * 初始化影子节点
   */
  setupShadow() {
    render(this.html(), this.shadowRoot);
    this.renderCss();
  }
  renderCss() {
    const stylesheet = new CSSStyleSheet();
    stylesheet.replaceSync(this.css());
    this.shadowRoot.adoptedStyleSheets = [stylesheet];
  }
  setProps() {
    this.props = Array.from(this.attributes).reduce((result, item) => {
      const { name, value } = item;
      result[name] = value;
      return result;
    }, {});
    this.renderCss();
  }
  css() {
    throw new Error("\u5FC5\u987B\u91CD\u5199\u7236\u7C7B css \u65B9\u6CD5");
  }
  html() {
    throw new Error("\u5FC5\u987B\u91CD\u5199\u7236\u7C7B html \u65B9\u6CD5");
  }
};

// src/event/createElement.ts
function createElement(tag, attributes, innerHTML, options) {
  const el = document.createElement(tag, options);
  if (!attributes)
    return el;
  for (const key in attributes)
    el.setAttribute(key, attributes[key]);
  if (innerHTML)
    el.innerHTML = innerHTML;
  return el;
}

// src/webComponent/crossImageElement.ts
var nameReg = /\w+-\w+/;
function crossImageElement(name = "img-cors") {
  if (!nameReg.test(name)) {
    return console.error(
      "crossImageElement: The naming must follow this format \u201Cxxx-xxx\u201D "
    );
  }
  window.customElements.define(name, ImageElement);
}
var ImageElement = class extends HTMLElement {
  shadow;
  width;
  height;
  constructor() {
    super();
    this.shadow = this.attachShadow({ mode: "open" });
    const iframe = createElement("iframe", {
      src: "about:blank",
      frameborder: "0"
    });
    iframe.onload = () => this.onIfrLoad(iframe);
    this.shadow.appendChild(iframe);
  }
  onIfrLoad(ifr) {
    const doc = ifr.contentWindow.document;
    doc.body.setAttribute("style", "margin:0;");
    const attributes = ["width", "height", "style", "class", "alt", "src"];
    const attrs = attributes.reduce((result, attribute) => {
      const attr = this.getAttribute(attribute);
      if (!attr)
        return result;
      return result += `${attribute}="${attr}" `;
    }, "");
    doc.body.innerHTML = `<img ${attrs}>`;
    doc.body.querySelector("img").onload = () => {
      this.width = ifr.width = this.width;
      this.height = ifr.height = this.height;
    };
  }
};

// src/webComponent/crossVideoElement.ts
var nameReg2 = /\w+-\w+/;
function crossVideoElement(name = "video-cors") {
  if (!nameReg2.test(name)) {
    return console.error(
      "crossImageElement: The naming must follow this format \u201Cxxx-xxx\u201D "
    );
  }
  window.customElements.define(name, VideoElement);
}
var VideoElement = class extends HTMLElement {
  shadow;
  width;
  height;
  constructor() {
    super();
    this.shadow = this.attachShadow({ mode: "open" });
    const iframe = createElement("iframe", {
      src: "about:blank",
      frameborder: "0"
    });
    iframe.onload = () => this.onIfrLoad(iframe, this.shadow.host.innerHTML);
    this.shadow.appendChild(iframe);
  }
  onIfrLoad(ifr, children) {
    const doc = ifr.contentWindow.document;
    doc.body.setAttribute("style", "margin:0;");
    const attributes = ["width", "height", "style", "class", "src", "controls"];
    const single = ["controls"];
    const attrs = attributes.reduce((result, attribute) => {
      const attr = this.getAttribute(attribute);
      if (single.includes(attribute))
        return result += `${attribute} `;
      if (!attr)
        return result;
      return result += `${attribute}="${attr}"`;
    }, "");
    doc.body.innerHTML = `<video ${attrs}>${children}</video>`;
    doc.body.querySelector("video").onload = () => {
      this.width = ifr.width = this.width;
      this.height = ifr.height = this.height;
    };
  }
};
export {
  Render,
  crossImageElement,
  crossVideoElement,
  render
};
